<?
////////////////////////////////////////////////////////////////////////
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
////////////////////////////////////////////////////////////////////////
class TransformRenderer extends CardRenderer {
	static private $settingSections;
    public $costColors1;
    public $pt2;
		
	public function render () {
		global $config, $dirDeckName, $FrameName, $ToTransform;
		$white = '255,255,255';
		echo $this->card . '...';
		$card = $this->card;
		$settings = $this->getSettings();
		$frame = $FrameName[strtolower($card->title)];
		$costColors = Card::getCostColors($card->cost);
		if($costColors == "" && $card->color == "Gld") $costColors = $ToTransform; //<----------------------
		$useMulticolorFrame = (strlen($costColors) > 1 && $costColors != "Art");
			
		$isPlaneswalker = 1;
		if (strpos($card->type, "Planeswalker") === false) {
			$isPlaneswalker = 0;
		}

		$canvas = imagecreatetruecolor(736, 1050);


/*********Planeswalker***************/
		if($isPlaneswalker == 1) {
			$useMulticolorFrame = strlen($costColors) == 2;
			// Art image.
			$this->drawArt($canvas, $card->artFileName, $settings['art.top.pwlkr'], $settings['art.left.pwlkr'], $settings['art.bottom.pwlkr'], $settings['art.right.pwlkr'], !$config['art.keep.aspect.ratio']);
	
			echo '.';
	
			// Background image.
			$borderImage = null;
			$greyTitleAndTypeOverlay = null;
			if ($card->isArtefact()) {
				$bgImage = @imagecreatefrompng("images/planeswalker/cards/Art_" . $frame . ".png");
			} else if ($useMulticolorFrame || $card->isDualManaCost()) {
				// Multicolor frame.
				if($settings['card.multicolor.gold.frame'])
					$bgImage = @imagecreatefrompng("images/planeswalker/cards/Gld$costColors" . "_" . $frame . ".png");
				else
					$bgImage = @imagecreatefrompng("images/planeswalker/cards/$costColors" . "_" . $frame . ".png");
				if (!$bgImage) error("Background image not found for color: $costColors" . "_$frame");
			} else {
				// Mono color frame.
				$bgImage = @imagecreatefrompng("images/planeswalker/cards/" . $card->color . "_" . $frame . ".png");
				if (!$bgImage) error("Background image not found for color \"" . $card->color . "_" . $frame . "\"");
			}
	
			imagecopy($canvas, $bgImage, 0, 0, 0, 0, 736, 1050);
			imagedestroy($bgImage);
	
			// Loyalty
			if ($card->pt) {
				$image = @imagecreatefrompng('images/planeswalker/loyalty/LoyaltyBegin.png');
				if (!$image) error("Loyalty image not found");
				imagecopy($canvas, $image, 605, 940, 0, 0, 127, 82);
				imagedestroy($image);
				$card->pt = (string)str_replace('/', '', $card->pt);
				$this->drawText($canvas, $settings['loyalty.starting.center.x'], $settings['loyalty.starting.center.y'], $settings['loyalty.starting.width'], $card->pt, $this->font('loyalty.starting', 'color:'.$white));
			}
			
			// Casting cost.
			if($frame == "Day")
				$costLeft = $this->drawCastingCost($canvas, $card->getCostSymbols(), $card->isDualManaCost() ? $settings['cost.top.dual.pwlkr'] : $settings['cost.top.pwlkr'], $settings['cost.right.pwlkr'], $settings['cost.size'], true);
			else
				$costLeft = $settings['cost.right'] - 5;
				
			echo '.';
	
			// Set and rarity.
			$rarityLeft = $this->drawRarity($canvas, $card->rarity, $card->set, $settings['rarity.right.pwlkr'], $settings['rarity.center.y.pwlkr'], $settings['rarity.height'], $settings['rarity.width'], false);
	
			// Card title.
			$this->drawText($canvas, $settings['title.x.pwlkr'], $settings['title.y.pwlkr'], $costLeft - $settings['title.x.pwlkr'], $card->getDisplayTitle(), $this->font('title', ($frame=="Night"?'color:'.$white:'')));
	
			echo '.';
	
			// Type.
			$this->drawText($canvas, $settings['type.x.' . $frame . '.pwlkr'], $settings['type.y.pwlkr'], $rarityLeft - $settings['type.x.' . $frame . '.pwlkr'], $card->type, $this->font('type', ($frame=="Night"?'color:'.$white:'')));
	
			// Legal text.
			if (!preg_match_all('/((\+|-)?([0-9XYZ]+): )?(.*?)(?=$|[\+|-]?[0-9XYZ]+:)/s', $card->legal, $matches)) error('Missing legality change from legal text: ' . $card->title);
//			print_r($matches);

			$text_offset=0;
			if(strlen($matches[0][0])<=1)
			{
				$text_offset=1;
			}
	
			$logaltyImage = null;
			$loyalty = array();
	
			//1
			$this->loyaltyIcon($matches[2][$text_offset+0], 1, $logaltyImage, $loyalty, $matches[3][$text_offset+0]);
			imagecopy($canvas, $logaltyImage, $loyalty['x'], $loyalty['y'], 0, 0, $loyalty['w'], $loyalty['h']);
			imagedestroy($logaltyImage);
	
			$this->drawText($canvas, $settings['loyalty.1.center.x'], $settings['loyalty.1.center.y'], $settings['loyalty.1.center.width'], (!strcmp($matches[1][0],"0: ")?"":substr($matches[1][$text_offset+0], 0, strrpos($matches[1][$text_offset+0], ":"))), $this->font('loyalty.change', 'color:'.$white));
			$this->drawLegalAndFlavorText($canvas, $settings['text.1.top.pwlkr'], $settings['text.1.left.pwlkr'], $settings['text.1.bottom.pwlkr'], $settings['text.1.right.pwlkr'], $matches[4][$text_offset+0], null, $this->font('text.pwlkr'), 0);
	
			//2
			$this->loyaltyIcon($matches[2][$text_offset+1], 2,  $logaltyImage, $loyalty, $matches[3][$text_offset+1]);
			imagecopy($canvas, $logaltyImage, $loyalty['x'], $loyalty['y'], 0, 0, $loyalty['w'], $loyalty['h']);
			imagedestroy($logaltyImage);
	
			$this->drawText($canvas, $settings['loyalty.2.center.x'], $settings['loyalty.2.center.y'], $settings['loyalty.2.center.width'], (!strcmp($matches[1][1],"0: ")?"":substr($matches[1][$text_offset+1], 0, strrpos($matches[1][$text_offset+1], ":"))), $this->font('loyalty.change', 'color:'.$white));
			$this->drawLegalAndFlavorText($canvas, $settings['text.2.top.pwlkr'], $settings['text.2.left.pwlkr'], $settings['text.2.bottom.pwlkr'], $settings['text.2.right.pwlkr'], $matches[4][$text_offset+1], null, $this->font('text.pwlkr'), 0);
	
			//3
			$this->loyaltyIcon($matches[2][$text_offset+2], 3,  $logaltyImage, $loyalty, $matches[3][$text_offset+2]);
			imagecopy($canvas, $logaltyImage, $loyalty['x'], $loyalty['y'], 0, 0, $loyalty['w'], $loyalty['h']);
			imagedestroy($logaltyImage);
	
			$this->drawText($canvas, $settings['loyalty.3.center.x'], $settings['loyalty.3.center.y'], $settings['loyalty.3.center.width'], (!strcmp($matches[1][2],"0: ")?"":substr($matches[1][$text_offset+2], 0, strrpos($matches[1][$text_offset+2], ":"))), $this->font('loyalty.change', 'color:'.$white));
			$this->drawLegalAndFlavorText($canvas, $settings['text.3.top.pwlkr'], $settings['text.3.left.pwlkr'], $settings['text.3.bottom.pwlkr'], $settings['text.3.right.pwlkr'], $matches[4][$text_offset+2], null, $this->font('text.pwlkr'), 10);
		
			// Artist and copyright.
			// The artist color is white if the frame behind it is black.
			$footerColor = '255,255,255';
			if ($card->artist) {
				if ($settings['card.artist.gears']) {
					$artistSymbol = '{gear}';
				} else {
					$artistSymbol = '{brush}';
				}
				$this->drawText($canvas, $settings['artist.x.pwlkr'], $settings['artist.y.pwlkr'], $settings['artist.width.pwlkr'], $artistSymbol . $card->artist, $this->font('artist.pwlkr', 'color:' . $footerColor));
			}
			if ($card->copyright) $this->drawText($canvas, $settings['copyright.x.pwlkr'], $settings['copyright.y.pwlkr'], null, $card->copyright, $this->font('copyright', 'color:' . $footerColor));
	
			//Art overlay
			$overlayFileName = preg_replace('/(.*)\.(jpg|png)/i','$1-overlay.png', $card->artFileName);
			if(file_exists($overlayFileName))
				$this->drawArt($canvas, $overlayFileName, $settings['art.top.pwlkr'], $settings['art.left.pwlkr'], $settings['art.bottom.pwlkr'], $settings['art.right.pwlkr'], !$config['art.keep.aspect.ratio']);
		}
/**********Other Cards***************/
		else {
			// Art image.
			$this->drawArt($canvas, $card->artFileName, $settings['art.top'], $settings['art.left'], $settings['art.bottom'], $settings['art.right'], !$config['art.keep.aspect.ratio']);
				
			echo '.';
	
			// Background image, border image, and grey title/type image.
			$borderImage = null;
					
			if ($card->isArtefact()) {
				$bgImage = @imagecreatefrompng("images/eighth/regular/cards/Art_" . $frame . ".png");
			} else if ($useMulticolorFrame) {
				// Multicolor frame.
				$bgImage = @imagecreatefrompng("images/eighth/regular/cards/$costColors" . "_" . $frame . ".png");
				if (!$bgImage) error("Background image not found for color: $costColors" . "_" . $frame . ".png");
			} else {
				// Mono color frame.
				$bgImage = @imagecreatefrompng("images/eighth/regular/cards/" . $card->color . "_" . $frame . ".png");
				if (!$bgImage) error("Background image not found for color \"" . $card->color . "_" . $frame . "\"");
			}
			
			imagecopy($canvas, $bgImage, 0, 0, 0, 0, 736, 1050);
			imagedestroy($bgImage);
	
			// Power / toughness.
			if ($card->pt) {
				if ($useMulticolorFrame)
					$image = @imagecreatefrompng("images/eighth/regular/pt/Gld" . ($frame=="Night"?"_Night":"") . '.png'); //substr($costColors, -1, 1)
				else
					$image = @imagecreatefrompng("images/eighth/regular/pt/" . $card->color . ($frame=="Night"?"_Night":"") . '.png');
				if (!$image) error("Power/toughness image not found for color: $color");
				imagecopy($canvas, $image, 0, 1050 - 162, 0, 0, 736, 162);
				imagedestroy($image);
				$this->drawText($canvas, $settings['pt.' . $frame . '.center.x'], $settings['pt.' . $frame . '.center.y'], $settings['pt.' . $frame . '.width'], $card->pt, $this->font('pt', ($frame=="Night"?'color:'.$white:'')));
			}
			if($frame == "Day") //<-----------------------------------------------
				if ($ToTransform) $this->drawText($canvas, $settings['pt.p.center.x'] - (strlen($ToTransform)>3?15:0), $settings['pt.p.center.y'], $settings['pt.p.width'], $ToTransform, $this->font('ptp'));
	
			// Casting cost.
			if($frame == "Day")
				$costLeft = $this->drawCastingCost($canvas, $card->getCostSymbols(), $card->isDualManaCost() ? $settings['cost.top.dual'] : $settings['cost.top'], $settings['cost.right'], $settings['cost.size'], true);
			else
				$costLeft = $settings['cost.right'] - 5;
	
			echo '.';
	
			// Set and rarity.
			if (!$card->isBasicLand() || $settings['card.basic.land.set.symbols'])
				$rarityLeft = $this->drawRarity($canvas, $card->rarity, $card->set, $settings['rarity.right'], $settings['rarity.center.y'], $settings['rarity.height'], $settings['rarity.width'], false);
			else
				$rarityLeft = $settings['rarity.right'];
	
			// Card title.
			$this->drawText($canvas, $settings['title.x'], $settings['title.y'], $costLeft - $settings['title.x'], $card->getDisplayTitle(), $this->font('title', ($frame=="Night"?'color:'.$white:'')));
	
			echo '.';
	
			// Type.
			$this->drawText($canvas, $settings['type.' . ($card->isArtefact()?"Day":$frame) . '.x'], $settings['type.' . $frame . '.y'], $rarityLeft - $settings['type.' . $frame . '.x'], $card->type, $this->font('type', ($frame=="Night"?'color:'.$white:'')));
	
			// Promo overlay
			if ($card->promo && !$card->isBasicLand()) {
				list($image, $width, $height) = getPNG('images/promo/' . $card->promo . '.png', 'Promo overlay image not found for: ' . $card->promo);
				imagecopy($canvas, $image, 359 - ($width / 2), 680, 0, 0, $width, $height);
				imagedestroy($image);
			}
			
			// Legal and flavor text.
			$heightAdjust = $card->pt ? $settings['text.pt.height.adjust'] : 0;
			$this->drawLegalAndFlavorText($canvas, $settings['text.top'], $settings['text.left'], $settings['text.bottom'], $settings['text.right'], $card->legal, $card->flavor, $this->font('text'), $heightAdjust);
	
			// Artist and copyright.
			// The artist color is white if the frame behind it is black.
			$footerColor = '0,0,0';
			if ($card->color == 'B' && !$card->isArtefact())
				$footerColor = '255,255,255';
			else if ($costColors == 'B' && !$card->isArtefact())
				$footerColor = '255,255,255';
			else if ($useMulticolorFrame) {
				// Only multicolor frames with a bottom left color of black should use a white footer.
				if ((strlen($costColors) <= 2 && substr($costColors, 0, 1) == 'B') || (strlen($costColors) >= 3 && substr($costColors, 2, 1) == 'B'))
					$footerColor = '255,255,255';
			}
			if ($card->artist) {
				if ($settings['card.artist.gears']) {
					$artistSymbol = '{gear}';
				} else {
					$artistSymbol = '{brush}';
				}
				$this->drawText($canvas, $settings['artist.x'], $settings['artist.y'], null, $artistSymbol . $card->artist, $this->font('artist', 'color:' . $footerColor));
			}
			if ($card->copyright) $this->drawText($canvas, $settings['copyright.x'], $settings['copyright.y'], null, $card->copyright, $this->font('copyright', 'color:' . $footerColor));
		}
		
		// print seperated images too
		if($config['output.split.cards.separated']) {
			$renderer->card = $card;
			$renderer->writer = $this;
			$outputName = $card->title;
			if ($card->pic) $outputName .= ' (' . $card->pic . ')';
			$outputName .= $config['output.suffix'];
			// Filenames can't contain ".
			if (strpos($outputName, '"') !== FALSE) $outputName = str_replace('"', '',  $outputName);
			
			$dirName = $dirDeckName;
			if($config['output.card.set.directories'] && !$this->writer->pagedOutput) $dirName .= $card->set . '/';
			
			$resizeWidth = $config['output.card.width'];
			if ($resizeWidth) $resizeHeight = $resizeWidth * (1050 / 736);
			if ($resizeWidth) {
				$image = imagecreatetruecolor($resizeWidth, $resizeHeight);
				imagecopyresampled($image, $canvas, 0, 0, 0, 0, $resizeWidth, $resizeHeight, 736, 1050);
				$canvas = $image;
				imagedestroy($image);
			}
			$this->writer->outputImage($canvas, $this->writer->getOutputFileName($dirName, $outputName));
		}
		
		echo "\n";
		return $canvas;
	}
	
	private function loyaltyIcon($sign, $slotNumber, &$logaltyImage, &$loyalty, $valor){
		if ($sign == '+'){
			$logaltyImage = @imagecreatefrompng('images/planeswalker/loyalty/LoyaltyUp.png');
			switch($slotNumber){
				case 1:
					$loyalty['y'] = 687;
					break;
				case 2:
					$loyalty['y'] = 785;
					break;
				case 3:
					$loyalty['y'] = 876;
					break;
			}
			$loyalty['x'] = 13;
			$loyalty['w'] = 97;
			$loyalty['h'] = 76;
		} else if ($sign == '-'){
			$logaltyImage = @imagecreatefrompng('images/planeswalker/loyalty/LoyaltyDown.png');
			switch($slotNumber){
				case 1:
					$loyalty['y'] = 698;
					break;
				case 2:
					$loyalty['y'] = 794;
					break;
				case 3:
					$loyalty['y'] = 887;
					break;
			}
			$loyalty['x'] = 13;
			$loyalty['w'] = 97;
			$loyalty['h'] = 75;
		} else if (($sign == '')&&($valor == '')){
			$logaltyImage = @imagecreatefrompng('images/planeswalker/loyalty/NoLoyalty.png');
				switch($slotNumber){
					case 1:
						$loyalty['y'] = 691;
						break;
					case 2:
						$loyalty['y'] = 789;
						break;
					case 3:
						$loyalty['y'] = 880;
						break;
				}
				$loyalty['x'] = 13;
				$loyalty['w'] = 97;
				$loyalty['h'] = 76;
			}else {
				$logaltyImage = @imagecreatefrompng('images/planeswalker/loyalty/LoyaltyZero.png');
				switch($slotNumber){
					case 1:
						$loyalty['y'] = 691;
						break;
					case 2:
						$loyalty['y'] = 789;
						break;
					case 3:
						$loyalty['y'] = 880;
						break;
				}
				$loyalty['x'] = 13;
				$loyalty['w'] = 98;
				$loyalty['h'] = 76;
			}

	}
	
	public function getSettings () {
		global $rendererSettings, $rendererSections;
		$settings = $rendererSettings['config/config-transform.txt'];
		$settings = array_merge($settings, $rendererSections['config/config-transform.txt']['fonts - regular']);
		return $settings;
	}
	
}

?>
